@@ -57,6 +57,7 @@
 #include "otrguiclient.h"
 #include "otrplugin.h"
 #include "privkeypopup.h"
+#include "smppopup.h"
 
 OtrlChatInterface *OtrlChatInterface::mSelf = 0;
 static OtrlUserState userstate;
@@ -148,7 +149,7 @@ static int display_otr_message( void *opdata, const char *accountname, const cha
 	Kopete::ContactPtrList list = session->members();		
 	for ( QPtrListIterator<Kopete::Contact> it( list ); Kopete::Contact *contact = it.current(); ++it ){
 		if( contact->contactId().compare( username ) == 0 ){
-			Kopete::Message msg( session->members().getFirst(), session->account()->myself(), QString( message ), Kopete::Message::Internal  );
+			Kopete::Message msg( session->members().getFirst(), session->account()->myself(), QString( message ), Kopete::Message::Internal );
 			msg.setBody( QString( message ), Kopete::Message::RichText );
 			session->appendMessage( msg );
 			return 0;
@@ -173,15 +174,9 @@ static void protocol_name_free(void *opdata, const char *protocol_name){
 
 static void new_fingerprint(void *opdata, OtrlUserState us, const char *accountname, const char *protocol, const char *username, unsigned char fingerprint[20]){
 	kdDebug() << "Received a new Fingerprint" << endl;
-	char human[45];
-	otrl_privkey_hash_to_human( human, fingerprint );
-	ConnContext *context = otrl_context_find( us, username, accountname, protocol, 0, NULL, NULL, NULL);
-	Fingerprint *fpPointer = otrl_context_find_fingerprint( context, fingerprint, 0, NULL );
-	if( OtrlChatInterface::self()->verifyQuestion( ((Kopete::ChatSession*)opdata), human ) ){
-		otrl_context_set_trust( fpPointer, "verified" );
-	} else {
-		otrl_context_set_trust( fpPointer, NULL );
-	}
+	Kopete::ChatSession *session= ((Kopete::ChatSession*)opdata);
+	Kopete::Message msg( session->members().getFirst(), session->account()->myself(), i18n("<b>Received a new fingerprint from <a>%1</a>. You should authenticate this contact.</b>").arg( session->members().getFirst()->contactId() ), Kopete::Message::Internal, Kopete::Message::RichText  );	
+	session->appendMessage( msg );
 }
 
 static void write_fingerprints(void *opdata){
@@ -230,13 +225,6 @@ static void still_secure(void *opdata, ConnContext *context, int is_reply){
 
 static void log_message(void *opdata, const char *message){
 	kdDebug() << "libotr: "<< message << endl;
-// setting gui icon to disconnected because libotr sends a heartbeat message when calling otrl_context_disconnect()
-// and this doesn't call gone_insecure... I also have never seen a heartbeat message not generated while disconnect.
-	Kopete::ChatSession *session= ((Kopete::ChatSession*)opdata);
-	Kopete::Message msg( session->members().getFirst(), session->account()->myself(), i18n("<b>%1</b> has ended the OTR session. You should do the same.").arg(session->members().getFirst()->contactId()) , Kopete::Message::Internal, Kopete::Message::RichText  );
-	session->appendMessage( msg );
-
-	OTRPlugin::plugin()->emitGoneSecure(((Kopete::ChatSession*)opdata), 3);
 }
 
 static OtrlMessageAppOps ui_ops = {
@@ -302,8 +290,90 @@ int OtrlChatInterface::decryptMessage( QString *msg, QString accountId,
 	int ignoremessage;
 	char *newMessage = NULL;
 	OtrlTLV *tlvs = NULL;
+	OtrlTLV *tlv = NULL;
+	ConnContext *context;
+	NextExpectedSMP nextMsg;
+
 
 	ignoremessage = otrl_message_receiving( userstate, &ui_ops, chatSession, accountId.latin1(), protocol.latin1(), contactId.latin1(), msg->latin1(), &newMessage, &tlvs, NULL, NULL );
+
+
+	tlv = otrl_tlv_find(tlvs, OTRL_TLV_DISCONNECTED);
+	if( tlv ){
+		Kopete::Message msg( chatSession->members().getFirst(), chatSession->account()->myself(), i18n("<b>%1</b> has ended the OTR session. You should do the same.").arg(chatSession->members().getFirst()->contactId()) , Kopete::Message::Internal, Kopete::Message::RichText  );
+		chatSession->appendMessage( msg );
+	
+		OTRPlugin::plugin()->emitGoneSecure( chatSession, 3 );
+	}
+
+	context = otrl_context_find( userstate, contactId.latin1(), accountId.latin1(), protocol.latin1(), 0, NULL, NULL, NULL);
+	if (context) {
+		nextMsg = context->smstate->nextExpected;
+
+		tlv = otrl_tlv_find(tlvs, OTRL_TLV_SMP1);
+		if (tlv) {
+			if (nextMsg != OTRL_SMP_EXPECT1){
+				abortSMP( context, chatSession );
+			} else {
+				SMPPopup *popup = new SMPPopup( chatSession->view()->mainWidget(), i18n("Enter authentication secret"),  Qt::WStyle_Dialog | Qt::WStyle_StaysOnTop, context, chatSession, false );
+				popup->show();
+			}
+		}
+		tlv = otrl_tlv_find(tlvs, OTRL_TLV_SMP2);
+		if (tlv) {
+			if (nextMsg != OTRL_SMP_EXPECT2)
+				abortSMP( context, chatSession );
+			else {
+				kdDebug() << "Update SMP state: 2 -> 3" << endl;
+				context->smstate->nextExpected = OTRL_SMP_EXPECT4;
+			}
+		}
+		tlv = otrl_tlv_find(tlvs, OTRL_TLV_SMP3);
+		if (tlv) {
+			if (nextMsg != OTRL_SMP_EXPECT3)
+				abortSMP( context, chatSession );
+			else {
+				if (context->active_fingerprint->trust && context->active_fingerprint->trust[0]) {
+					Kopete::Message msg( chatSession->members().getFirst(), chatSession->account()->myself(), i18n("<b>Authentication successful. The conversation is now secure!</b>"), Kopete::Message::Internal, Kopete::Message::RichText );
+					chatSession->appendMessage( msg );
+					OTRPlugin::plugin()->emitGoneSecure( chatSession, 2 );
+				} else {
+					Kopete::Message msg( chatSession->members().getFirst(), chatSession->account()->myself(), i18n("<b>Authentication failed. The conversation is now insecure!</b>"), Kopete::Message::Internal, Kopete::Message::RichText );
+					chatSession->appendMessage( msg );
+					OTRPlugin::plugin()->emitGoneSecure( chatSession, 1 );
+				}
+
+				context->smstate->nextExpected = OTRL_SMP_EXPECT1;
+			}
+		}
+		tlv = otrl_tlv_find(tlvs, OTRL_TLV_SMP4);
+		if (tlv) {
+			if (nextMsg != OTRL_SMP_EXPECT4)
+				abortSMP( context, chatSession );
+			else {
+				if (context->active_fingerprint->trust && context->active_fingerprint->trust[0]) {
+					Kopete::Message msg( chatSession->members().getFirst(), chatSession->account()->myself(), i18n("<b>Authentication successful. The conversation is now secure!</b>"), Kopete::Message::Internal, Kopete::Message::RichText );
+					chatSession->appendMessage( msg );
+					OTRPlugin::plugin()->emitGoneSecure( chatSession, 2 );
+				} else {
+					Kopete::Message msg( chatSession->members().getFirst(), chatSession->account()->myself(), i18n("<b>Authentication failed. The conversation is now insecure!</b>"), Kopete::Message::Internal, Kopete::Message::RichText );
+					chatSession->appendMessage( msg );
+					OTRPlugin::plugin()->emitGoneSecure( chatSession, 1 );
+				}
+				context->smstate->nextExpected = OTRL_SMP_EXPECT1;
+			}
+		}
+		tlv = otrl_tlv_find(tlvs, OTRL_TLV_SMP_ABORT);
+		if (tlv) {
+			Kopete::Message msg( chatSession->members().getFirst(), chatSession->account()->myself(), i18n("<b>Authentication error!</b>"), Kopete::Message::Internal, Kopete::Message::RichText );
+			chatSession->appendMessage( msg );
+			context->smstate->nextExpected = OTRL_SMP_EXPECT1;
+		}
+	
+		otrl_tlv_free(tlvs);
+	}
+	
+
 	// message is now decrypted or is a Plaintext message and ready to deliver
 	if( !ignoremessage ){
 		// message is decrypted
@@ -412,7 +482,18 @@ QString  OtrlChatInterface::formatContact(QString contactId){
 	return contactId;
 }
 
-void OtrlChatInterface::verifyFingerprint( Kopete::ChatSession *session, bool trust ){
+void OtrlChatInterface::verifyFingerprint( Kopete::ChatSession *session ){
+	ConnContext *context;
+
+	context = otrl_context_find( userstate, session->members().getFirst()->contactId().latin1(), session->account()->accountId().latin1(), session->protocol()->displayName().latin1(), 0, NULL, NULL, NULL);
+
+	SMPPopup *popup = new SMPPopup( session->view()->mainWidget(), i18n("Enter authentication secret"),  Qt::WStyle_Dialog | Qt::WStyle_StaysOnTop, context, session, true );
+	popup->show();
+
+
+}
+
+void OtrlChatInterface::setTrust( Kopete::ChatSession *session, bool trust ){
 	Fingerprint *fingerprint;
 
 	fingerprint = findFingerprint( session->members().getFirst()->contactId() );
@@ -428,7 +509,6 @@ void OtrlChatInterface::verifyFingerprint( Kopete::ChatSession *session, bool tr
 	} else {
 		kdDebug() << "could not find fingerprint" << endl;
 	}
-
 }
 
 Fingerprint *OtrlChatInterface::findFingerprint( QString account ){
@@ -542,7 +622,7 @@ void OtrlChatInterface::checkFilePermissions( QString file ){
 
 }
 
-bool OtrlChatInterface::verifyQuestion( Kopete::ChatSession *session, QString fingerprint ){
+/*bool OtrlChatInterface::verifyQuestion( Kopete::ChatSession *session, QString fingerprint ){
 	kdDebug() << "searching for Fingerprint" << endl;
 
 	if( fingerprint != NULL ){
@@ -561,7 +641,32 @@ bool OtrlChatInterface::verifyQuestion( Kopete::ChatSession *session, QString fi
 	}
 	return false;	
 }
+*/
 
+/****************** SMP implementations ****************/
+
+void OtrlChatInterface::abortSMP( ConnContext *context, Kopete::ChatSession *session ){
+	otrl_message_abort_smp( userstate, &ui_ops, session, context);
+	if (context->active_fingerprint->trust && !context->active_fingerprint->trust[0]) {
+		OTRPlugin::plugin()->emitGoneSecure( session, 1 );
+		Kopete::Message msg( session->members().getFirst(), session->account()->myself(), i18n("<b>Authentication aborded. The conversation is now insecure!</b>"), Kopete::Message::Internal, Kopete::Message::RichText );
+		session->appendMessage( msg );
+	}
+}
+
+void OtrlChatInterface::respondSMP( ConnContext *context, Kopete::ChatSession *session, QString secret, bool initiate ){
+	if( initiate ){
+		context = otrl_context_find( userstate, session->members().getFirst()->contactId().latin1(), session->account()->accountId().latin1(), session->protocol()->displayName().latin1(), 0, NULL, NULL, NULL);
+		otrl_message_initiate_smp( userstate, &ui_ops, session, context, (unsigned char*)secret.latin1(), secret.length() );
+
+		
+	} else {
+		otrl_message_respond_smp( userstate, &ui_ops, session, context, (unsigned char*)secret.latin1(), secret.length());
+	}
+
+	Kopete::Message msg( session->members().getFirst(), session->account()->myself(), i18n("<b>Authenticating contact...</b>"), Kopete::Message::Internal, Kopete::Message::RichText );
+	session->appendMessage( msg );
+}
 
 /****************** KeyGenThread *******************/
 
